import 'dart:html';
import 'package:flutter/material.dart';
import 'package:flutter/widgets.dart';
import 'package:nui_miniprogram_web/jsengine/nui_miniprogram_web_jsengine.dart';
import 'package:nui_miniprogram_web/model/nui_miniprogram_webroute.dart';

class RoutingData {
  final String route;
  final Map<String, String> _queryParameters;
  RoutingData({this.route, Map<String, String> queryParameters}) : _queryParameters = queryParameters;
  operator [](String key) => _queryParameters[key];
}

extension StringExtension on String {
  RoutingData get getRoutingData {
    var uriData = Uri.parse(this);
    print('[NUIMiniProgramWebApp] queryParameters: ${uriData.queryParameters} path: ${uriData.path}');
    return RoutingData(
      queryParameters: uriData.queryParameters,
      route: uriData.path,
    );
  }
}

final GlobalKey<NavigatorState> _webAppKey = new GlobalKey<NavigatorState>();

class NUIMiniProgramWebApp extends StatefulWidget {
  NUIMiniProgramWebApp({@required this.initialRoute, this.routes, this.theme, this.initializeOnInit: true, this.background : Colors.white, this.builder});
  final NUIMiniProgramWebRoute initialRoute;
  final List<NUIMiniProgramWebRoute> routes;
  final TransitionBuilder builder;
  final Color background;
  final ThemeData theme;
  final bool initializeOnInit;

  static GlobalKey<NavigatorState> get globalKey{
    return _webAppKey;
  }
  
  @override
  _NUIMiniProgramWebAppState createState() => _NUIMiniProgramWebAppState();
}

class _NUIMiniProgramWebAppState extends NUIMiniProgramState<NUIMiniProgramWebApp> {

  @override
  void initState(){
    _initializeOnInit = widget.initializeOnInit ?? true;
    super.initState();
  }

  @override
  Widget build(BuildContext context) {
    // final routes = _definedRoutes();
    return MaterialApp(
      navigatorKey: _webAppKey,
      builder: widget.builder ?? null,
      onGenerateRoute: _generateRoute,
      // routes: _getRoutes(),
      theme: widget.theme,
      color: widget.background ?? Colors.white,
    );
  }

  Route<dynamic> _generateRoute(RouteSettings settings) {
    if(settings.name == "/") {
      final child = widget.initialRoute.child(context);
      return widget.initialRoute.routeBuilder == null ? MaterialPageRoute(builder: (context) => widget.initialRoute.child(context), settings: settings) : widget.initialRoute.routeBuilder(child);
    }

    final routes = _definedRoutes();
    var routingData = settings.name.getRoutingData;

    final definedPaths = routes.keys.toList();
    for(var path in definedPaths){
      if(path.isEmpty || path == "/") continue;
      if(routingData.route.contains(path)){
        //Found a matching wildcard path, process them
        final route = routes[path];
        final splitPaths = routingData.route.split(path);
        final pathVariable = splitPaths[1];
        final child = route.child(context, variable: pathVariable);

        return route.routeBuilder == null ? MaterialPageRoute(builder: (context) => child, settings: settings) : route.routeBuilder(child);
      }
    }

    final child = widget.initialRoute.child(context);
    return widget.initialRoute.routeBuilder == null ? MaterialPageRoute(builder: (context) => widget.initialRoute.child(context), settings: settings) : widget.initialRoute.routeBuilder(child);
  }

  Map<String, NUIMiniProgramWebRoute> _definedRoutes(){
    final routeMap = Map<String, NUIMiniProgramWebRoute>();
    widget.routes?.forEach((element) {
      routeMap[element.route] = element;
    });
    return routeMap;
  }

  Map<String, Widget Function(BuildContext)> _getRoutes(){
    final routeMap = Map<String, Widget Function(BuildContext)>();
    routeMap["/"] = (context) => Builder(
      builder: (context) {
        jsEngine.initInternalChannelsForWeb(context);
        return widget.initialRoute.child(context);
      },
    );
    widget.routes?.forEach((element) {
      routeMap[element.route] = (context) => element.child(context);
    });
    return routeMap;
  }
}

abstract class NUIMiniProgramState<T extends StatefulWidget> extends State<T>{
  NUIMiniProgramWebJSEngine _jsEngine;
  bool _initializeOnInit = true;
  Duration _delayForInit;
  Duration _initializeMaxDelay = Duration(seconds: 1);

  void setDelayForInit(Duration delay){
    _delayForInit = delay;
  }

  void setInitializeMaxDelay(Duration delay){
    _initializeMaxDelay = delay;
  }

  NUIMiniProgramWebJSEngine get jsEngine{
    if(_jsEngine == null){
      _jsEngine = NUIMiniProgramWebJSEngine.from(window)..initCrossEventListener((message) {
        return null;
      });
      if(_initializeOnInit == true) {
        if(_delayForInit == null) {
          _jsEngine.sendInitializedEvent(maximumDelay: _initializeMaxDelay);
        }
        else{
          Future.delayed(_delayForInit, (){
            _jsEngine.sendInitializedEvent(maximumDelay: _initializeMaxDelay);
          });
        }
      }
    }
    return _jsEngine;
  }

  @override
  void initState(){
    super.initState();
    jsEngine;
  }
}
